/*
 * 

 RasterSource.java

 Copyright 2004 KUBO Hiroya (hiroya@sfc.keio.ac.jp).

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
package net.sf.sqs_xml.image;

import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

public class RasterSource {
	final RasterSource source;
	final Raster raster;
   BufferedImage image = null;
    
	int width = 0;
	int height = 0;

	int threshold = 200;
	
	public static final int BLACK = 0;
	public static final int[] BLACK_COLOR = {0, 0, 0};
	public static final int WHITE = 0x00ffffff;
	public static final int[] WHITE_COLOR = {255, 255, 255};
	
	public RasterSource() {
			this.source = null;
			this.raster = null;
	}

    
    
	public RasterSource(BufferedImage image) {
	    this.image = image; 
		this.width = image.getWidth();
		this.height = image.getHeight();
		this.source = null;
		this.raster = image.getData();
	}
    
	public RasterSource(Raster raster) {
		this.width = raster.getWidth();
		this.height = raster.getHeight();
		this.source = null;
		this.raster = raster;
	}

	public RasterSource(RasterSource source) {
		this.width = source.getWidth();
		this.height = source.getHeight();
		this.source = source;
		this.raster = null;
	}

	public void setThreshold(int threshold){
	    this.threshold = threshold;
	}
	
	public int getThreshold(){
	    return this.threshold;
	}
	
    public final int getWidth() {
		return width;
	}

	public final int getHeight() {
		return height;
	}

	/*
	int getAntialiasedPixel(double x, double y) {
		double xfloor = Math.floor(x);
		double yfloor = Math.floor(y);
		double xceil = Math.ceil(x);
		double yceil = Math.ceil(y);

		int p1 = getPixel((int) xfloor, (int) yfloor);
		int p2 = getPixel((int) xceil, (int) yfloor);
		int p3 = getPixel((int) xfloor, (int) yceil);
		int p4 = getPixel((int) xceil, (int) yceil);

		return (int)
				((p1 & 0xff) * (xceil - x) * (yceil - y)
					+ (p2 & 0xff) * (x - xfloor) * (yceil - y)
					+ (p3 & 0xff) * (xceil - x) * (y - yfloor)
					+ (p4 & 0xff) * (x - xfloor) * (y - yfloor));

		switch (getMode()) {
			case GRAYSCALE_MODE :
				return (int)
					((p1 & 0xff) * (xceil - x) * (yceil - y)
						+ (p2 & 0xff) * (x - xfloor) * (yceil - y)
						+ (p3 & 0xff) * (xceil - x) * (y - yfloor)
						+ (p4 & 0xff) * (x - xfloor) * (y - yfloor));
			case RGB_MODE :
			default :
				int ret = 0;
				for (int bitshift = 0; bitshift < 32; bitshift += 8) {
					ret
						|= (int) (((p1 << bitshift) & 0xff)
							* (xceil - x)
							* (yceil - y)
							+ ((p2 << bitshift) & 0xff)
								* (x - xfloor)
								* (yceil - y)
							+ ((p3 << bitshift) & 0xff)
								* (xceil - x)
								* (y - yfloor)
							+ ((p4 << bitshift) & 0xff)
								* (x - xfloor)
								* (y - yfloor))
						>> bitshift;

				}
				return ret;
		}

	}
		*/
		
	public final boolean isBlack(int color) {
	    return isBlack((color&0x00ff0000)>>16, (color&0x0000ff00)>>8, (color&0x00000ff));
	}
    
    public final boolean isBlack(int r, int g, int b) {
        return r < threshold || g < threshold || b < threshold;
    }

	public boolean isBlack(int x, int y) {
		return isBlack(getColor(x, y));
	}
	
	public int getColor(Point2D p) {
	    return getColor((int)p.getX(), (int)p.getY());
	}
	
	public int getColor(int x, int y) {
		if (raster != null) {
			if(x < 0 || width <= x){
				return BLACK;
			}else{
				try{
				    return image.getRGB(x, y);
				}catch(IndexOutOfBoundsException ex){
					return BLACK;
				}
			}
		}else{
			return source.getColor(x, y);
		}
	}
    
	public BufferedImage getImage(){
	    return image;
    }
    
	public Raster getRaster(){
	    if(raster != null){
	        return raster;
	    }else{
	        return source.getRaster();
	    }
	}

	public WritableRaster createCompatibleWritableRaster(int w, int h){
		if(raster != null){
		    return raster.createCompatibleWritableRaster(w, h);
		}else{
			 return source.createCompatibleWritableRaster(w, h);
		}
	}
	
	public int getNumBands(){
	    if(source == null){
	        return raster.getNumBands();
	    }else{
	        return source.getNumBands();
	    }
	}
	
	public RasterSource getSource(){
	    if(source == null){
	        return this;
	    }else{
	        return source;
	    }
	}
	
	public static BufferedImage createRGBImage(BufferedImage src) {
        if(src.getColorModel() instanceof IndexColorModel){
            BufferedImage image = new BufferedImage(src.getWidth(), src.getHeight(), BufferedImage.TYPE_INT_RGB);
            int color;
            for(int y=0; y < src.getHeight(); y++){
                 for(int x=0; x < src.getWidth(); x++){
                     color = src.getRGB(x, y);
                     image.setRGB(x, y, color);
                    }
            }
            return image;
        }
        return src;
    }

}
