/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package net.sf.sqs_xml.reader.model;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>ReadingState</b> is generated from reader.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="readingState" ns="http://sqs.cmr.sfc.keio.ac.jp/2004/reader">
 *     <attribute name="imagedir"><text/></attribute>
 *     <attribute name="pdffile"><text/></attribute>
 *     <attribute name="start">
 *        <data type="dateTime"/>              
 *     </attribute>                              
 *     
 *     <attribute name="end">
 *        <data type="dateTime"/>              
 *     </attribute>                              
 *     
 * 
 *     <optional>
 *       <ref name="ImageFiles"/>
 *     </optional>
 * 
 *     <optional>
 *       <element name="answers" ns="http://sqs.cmr.sfc.keio.ac.jp/2004/reader">
 *         <empty/>
 *       </element>
 *    </optional>
 *    </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="readingState" ns="http://sqs.cmr.sfc.keio.ac.jp/2004/reader"&gt;
 *     &lt;attribute name="imagedir"&gt;&lt;text/&gt;&lt;/attribute&gt;
 *     &lt;attribute name="pdffile"&gt;&lt;text/&gt;&lt;/attribute&gt;
 *     &lt;attribute name="start"&gt;
 *        &lt;data type="dateTime"/&gt;              
 *     &lt;/attribute&gt;                              
 *     
 *     &lt;attribute name="end"&gt;
 *        &lt;data type="dateTime"/&gt;              
 *     &lt;/attribute&gt;                              
 *     
 * 
 *     &lt;optional&gt;
 *       &lt;ref name="ImageFiles"/&gt;
 *     &lt;/optional&gt;
 * 
 *     &lt;optional&gt;
 *       &lt;element name="answers" ns="http://sqs.cmr.sfc.keio.ac.jp/2004/reader"&gt;
 *         &lt;empty/&gt;
 *       &lt;/element&gt;
 *    &lt;/optional&gt;
 *    &lt;/element&gt;</pre>
 *
 * @version reader.rng (Fri Aug 06 03:48:33 GMT+09:00 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class ReadingState implements java.io.Serializable, Cloneable, IRNSContainer, IRNode {
    private RNSContext rNSContext_ = new RNSContext(this, "http://sqs.cmr.sfc.keio.ac.jp/2004/reader");
    private String imagedir_;
    private String pdffile_;
    private java.sql.Timestamp start_;
    private java.sql.Timestamp end_;
    private ImageFiles imageFiles_;
    private String answers_;
    private Element xmlElement;
    private IRNode parentRNode_;

    /**
     * Creates a <code>ReadingState</code>.
     *
     */
    public ReadingState() {
        imagedir_ = "";
        pdffile_ = "";
        start_ = URelaxer.makeAutoSQLTimestamp();
        end_ = URelaxer.makeAutoSQLTimestamp();
    }

    /**
     * Creates a <code>ReadingState</code>.
     *
     * @param source
     */
    public ReadingState(ReadingState source) {
        setup(source);
    }

    /**
     * Creates a <code>ReadingState</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public ReadingState(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>ReadingState</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public ReadingState(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>ReadingState</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public ReadingState(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>ReadingState</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>ReadingState</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>ReadingState</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>ReadingState</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>ReadingState</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>ReadingState</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ReadingState(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>ReadingState</code> by the ReadingState <code>source</code>.
     *
     * @param source
     */
    public void setup(ReadingState source) {
        int size;
        imagedir_ = source.imagedir_;
        pdffile_ = source.pdffile_;
        start_ = source.start_;
        end_ = source.end_;
        if (source.imageFiles_ != null) {
            setImageFiles((ImageFiles)source.getImageFiles().clone());
        }
        answers_ = source.answers_;
    }

    /**
     * Initializes the <code>ReadingState</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>ReadingState</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>ReadingState</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        xmlElement = element;
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        imagedir_ = URelaxer.getAttributePropertyAsString(element, "imagedir");
        pdffile_ = URelaxer.getAttributePropertyAsString(element, "pdffile");
        start_ = URelaxer.getAttributePropertyAsSQLTimestamp(element, "start");
        end_ = URelaxer.getAttributePropertyAsSQLTimestamp(element, "end");
        if (ImageFiles.isMatch(stack)) {
            setImageFiles(new ImageFiles(stack));
        }
        answers_ = URelaxer2.getElementPropertyAsStringByStack(stack, "http://sqs.cmr.sfc.keio.ac.jp/2004/reader", "answers");
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new ReadingState((ReadingState)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://sqs.cmr.sfc.keio.ac.jp/2004/reader", "readingState");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.imagedir_ != null) {
            URelaxer.setAttributePropertyByString(element, "imagedir", this.imagedir_);
        }
        if (this.pdffile_ != null) {
            URelaxer.setAttributePropertyByString(element, "pdffile", this.pdffile_);
        }
        if (this.start_ != null) {
            URelaxer.setAttributePropertyBySQLTimestamp(element, "start", this.start_);
        }
        if (this.end_ != null) {
            URelaxer.setAttributePropertyBySQLTimestamp(element, "end", this.end_);
        }
        if (this.imageFiles_ != null) {
            this.imageFiles_.makeElement(element);
        }
        if (this.answers_ != null) {
            URelaxer2.setElementPropertyByString(element, "http://sqs.cmr.sfc.keio.ac.jp/2004/reader", "answers", this.answers_, rNSContext_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>ReadingState</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>ReadingState</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>ReadingState</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>ReadingState</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>ReadingState</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>ReadingState</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>imagedir</b>.
     *
     * @return String
     */
    public String getImagedir() {
        return (imagedir_);
    }

    /**
     * Sets the String property <b>imagedir</b>.
     *
     * @param imagedir
     */
    public void setImagedir(String imagedir) {
        this.imagedir_ = imagedir;
    }

    /**
     * Gets the String property <b>pdffile</b>.
     *
     * @return String
     */
    public String getPdffile() {
        return (pdffile_);
    }

    /**
     * Sets the String property <b>pdffile</b>.
     *
     * @param pdffile
     */
    public void setPdffile(String pdffile) {
        this.pdffile_ = pdffile;
    }

    /**
     * Gets the java.sql.Timestamp property <b>start</b>.
     *
     * @return java.sql.Timestamp
     */
    public java.sql.Timestamp getStart() {
        return (start_);
    }

    /**
     * Sets the java.sql.Timestamp property <b>start</b>.
     *
     * @param start
     */
    public void setStart(java.sql.Timestamp start) {
        this.start_ = start;
    }

    /**
     * Gets the java.sql.Timestamp property <b>end</b>.
     *
     * @return java.sql.Timestamp
     */
    public java.sql.Timestamp getEnd() {
        return (end_);
    }

    /**
     * Sets the java.sql.Timestamp property <b>end</b>.
     *
     * @param end
     */
    public void setEnd(java.sql.Timestamp end) {
        this.end_ = end;
    }

    /**
     * Gets the ImageFiles property <b>ImageFiles</b>.
     *
     * @return ImageFiles
     */
    public ImageFiles getImageFiles() {
        return (imageFiles_);
    }

    /**
     * Sets the ImageFiles property <b>ImageFiles</b>.
     *
     * @param imageFiles
     */
    public void setImageFiles(ImageFiles imageFiles) {
        this.imageFiles_ = imageFiles;
        if (imageFiles != null) {
            imageFiles.rSetParentRNode(this);
        }
    }

    /**
     * Gets the String property <b>answers</b>.
     *
     * @return String
     */
    public String getAnswers() {
        return (answers_);
    }

    /**
     * Sets the String property <b>answers</b>.
     *
     * @param answers
     */
    public void setAnswers(String answers) {
        this.answers_ = answers;
    }

    /**
     * Gets the element to be used in the object construction.
     *
     * @return Element
     */
    public Element rGetElement() {
        return (xmlElement);
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://sqs.cmr.sfc.keio.ac.jp/2004/reader");
        buffer.append("<");
        URelaxer.makeQName(prefix, "readingState", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (imagedir_ != null) {
            buffer.append(" ");
            buffer.append("imagedir");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getImagedir())));
            buffer.append("\"");
        }
        if (pdffile_ != null) {
            buffer.append(" ");
            buffer.append("pdffile");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getPdffile())));
            buffer.append("\"");
        }
        if (start_ != null) {
            buffer.append(" ");
            buffer.append("start");
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getStart()));
            buffer.append("\"");
        }
        if (end_ != null) {
            buffer.append(" ");
            buffer.append("end");
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getEnd()));
            buffer.append("\"");
        }
        buffer.append(">");
        if (imageFiles_ != null) {
            imageFiles_.makeTextElement(buffer);
        }
        if (answers_ != null) {
            buffer.append("<");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.append(">");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getAnswers())));
            buffer.append("</");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.append(">");
        }
        buffer.append("</");
        URelaxer.makeQName(prefix, "readingState", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://sqs.cmr.sfc.keio.ac.jp/2004/reader");
        buffer.write("<");
        URelaxer.makeQName(prefix, "readingState", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (imagedir_ != null) {
            buffer.write(" ");
            buffer.write("imagedir");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getImagedir())));
            buffer.write("\"");
        }
        if (pdffile_ != null) {
            buffer.write(" ");
            buffer.write("pdffile");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getPdffile())));
            buffer.write("\"");
        }
        if (start_ != null) {
            buffer.write(" ");
            buffer.write("start");
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getStart()));
            buffer.write("\"");
        }
        if (end_ != null) {
            buffer.write(" ");
            buffer.write("end");
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getEnd()));
            buffer.write("\"");
        }
        buffer.write(">");
        if (imageFiles_ != null) {
            imageFiles_.makeTextElement(buffer);
        }
        if (answers_ != null) {
            buffer.write("<");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.write(">");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getAnswers())));
            buffer.write("</");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.write(">");
        }
        buffer.write("</");
        URelaxer.makeQName(prefix, "readingState", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://sqs.cmr.sfc.keio.ac.jp/2004/reader");
        buffer.print("<");
        URelaxer.makeQName(prefix, "readingState", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (imagedir_ != null) {
            buffer.print(" ");
            buffer.print("imagedir");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getImagedir())));
            buffer.print("\"");
        }
        if (pdffile_ != null) {
            buffer.print(" ");
            buffer.print("pdffile");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getPdffile())));
            buffer.print("\"");
        }
        if (start_ != null) {
            buffer.print(" ");
            buffer.print("start");
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getStart()));
            buffer.print("\"");
        }
        if (end_ != null) {
            buffer.print(" ");
            buffer.print("end");
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getEnd()));
            buffer.print("\"");
        }
        buffer.print(">");
        if (imageFiles_ != null) {
            imageFiles_.makeTextElement(buffer);
        }
        if (answers_ != null) {
            buffer.print("<");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.print(">");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getAnswers())));
            buffer.print("</");
            URelaxer.makeQName(prefix, "answers", buffer);
            buffer.print(">");
        }
        buffer.print("</");
        URelaxer.makeQName(prefix, "readingState", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getImagedirAsString() {
        return (URelaxer.getString(getImagedir()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPdffileAsString() {
        return (URelaxer.getString(getPdffile()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getStartAsString() {
        return (URelaxer.getString(getStart()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getEndAsString() {
        return (URelaxer.getString(getEnd()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAnswersAsString() {
        return (URelaxer.getString(getAnswers()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setImagedirByString(String string) {
        setImagedir(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPdffileByString(String string) {
        setPdffile(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setStartByString(String string) {
        setStart(URelaxer.getSQLTimestamp(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setEndByString(String string) {
        setEnd(URelaxer.getSQLTimestamp(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAnswersByString(String string) {
        setAnswers(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (imageFiles_ != null) {
            classNodes.add(imageFiles_);
        }
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>ReadingState</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://sqs.cmr.sfc.keio.ac.jp/2004/reader", "readingState")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "imagedir")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "pdffile")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "start")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "end")) {
            return (false);
        }
        $match$ = true;
        if (ImageFiles.isMatchHungry(target)) {
        }
        child = target.peekElement();
        if (child != null) {
            if (URelaxer2.isTargetElement(child, "http://sqs.cmr.sfc.keio.ac.jp/2004/reader", "answers")) {
                target.popElement();
            }
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>ReadingState</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>ReadingState</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
